@@ -7,8 +7,7 @@ void *input_sndio(void *data) {
     struct audio_data *audio = (struct audio_data *)data;
     struct sio_par par;
     struct sio_hdl *hdl;
-    int16_t buf[256];
-    unsigned int channels;
+    int16_t buf[audio->FFTtreblebufferSize * 2];
 
     sio_initpar(&par);
     par.sig = 1;
@@ -34,26 +33,16 @@ void *input_sndio(void *data) {
         exit(EXIT_FAILURE);
     }
 
-    uint16_t frames = (sizeof(buf) / sizeof(buf[0])) / channels;
+    uint16_t frames = (sizeof(buf) / sizeof(buf[0])) / 2;
     while (audio->terminate != 1) {
         if (sio_read(hdl, buf, sizeof(buf)) == 0) {
             fprintf(stderr, __FILE__ ": sio_read() failed: %s\n", strerror(errno));
             exit(EXIT_FAILURE);
         }
 
-        write_to_fftw_input_buffers(buf, frames, audio);
-        /*
-                        for (i = 0; i < sizeof(buf)/sizeof(buf[0]); i += 2) {
-                                if (par.rchan == 1) {
-                                        // sndiod has already taken care of averaging the samples
-                                        audio->audio_out_l[n] = buf[i];
-                                } else if (par.rchan == 2) {
-                                        audio->audio_out_l[n] = buf[i];
-                                        audio->audio_out_r[n] = buf[i + 1];
-                                }
-                                n = (n + 1) % audio->FFTbufferSize;
-                        }
-        */
+        pthread_mutex_lock(&lock);
+        write_to_fftw_input_buffers(frames, buf, audio);
+        pthread_mutex_unlock(&lock);
     }
 
     sio_stop(hdl);
